###########################################################################
##  Kroller - v0.8
###########################################################################
##  
##  Code by:
##    Brandon Zehm <caspian at dotconf.net>
##  
##  Icon set arranged by:
##    Adam Duskett
##  
##  Kroller is an extension for SuperKaramba to make a MacOS-X like
##  menu bar.
##  It was originally based on tuxbar, but now only a few lines of
##  code from it remain.
##  
##  This SuperKaramba extension was designed to be flexible.
##  You can add new icons, or change existing ones by simply
##  editing the lines in the "User Configurable Section" in this
##  file.  You can use icons of any size you want.  All you need
##  to do is make sure that all the "small" icons are the same
##  size, and all the "big" icons are the same size, and you set
##  those sizes in the imageSize and imageSizeBig variables.
##
##  Feel free to email me with comments or suggestions!
##
###########################################################################
##
## To Do 
## - shring icons slightly or somehting when clicked
## - a k launch bar?
##
##
###########################################################################


## This import statement allows access to the karamba functions
import karamba

## Define some global variables
images=[]
imagesBig=[]
imageFiles=[]
imageFilesBig=[]
commands=[]




  ##                         ##
 ##                           ##
##  User Configurable Section  ##
 ##                           ##
  ##                         ##


## Sizes of your icons (icons should always be square, so in my case 32 means 32x32 pixels)
imageSize = 32
imageSizeBig = 128


## Specify paths to small icons
imageFiles.append("icons/home.png")
imageFiles.append("icons/konsole.png")
imageFiles.append("icons/control.png")
imageFiles.append("icons/email.png")
imageFiles.append("icons/gaim.png")
imageFiles.append("icons/konqueror.png")
imageFiles.append("icons/mozilla.png")
imageFiles.append("icons/xmms.png")


## Specify paths to large icons
imageFilesBig.append("icons/home2.png")
imageFilesBig.append("icons/konsole2.png")
imageFilesBig.append("icons/control2.png")
imageFilesBig.append("icons/email2.png")
imageFilesBig.append("icons/gaim2.png")
imageFilesBig.append("icons/konqueror2.png")
imageFilesBig.append("icons/mozilla2.png")
imageFilesBig.append("icons/xmms2.png")


## Specify commands to run for each icon
commands.append("kfmclient openProfile filemanagement")
commands.append("konsole")
commands.append("kcontrol")
commands.append("kmail")
commands.append("gaim")
commands.append("kfmclient openProfile webbrowsing")
commands.append("mozilla")
commands.append("xmms")


## To enable debugging set to 1 and run karamba from a konsole
debug = 0

  
  ##     ##                         ##
 ##     ##                           ##
##     ##  User Configurable Section  ##
 ##   ##                             ##
  ## ##                             ##




## Calculate some other important stuff
numIcons = len(imageFiles)
iconSeparation = (imageSizeBig / 2)                      ## The size of the "cells" where icons go
centerSmallIcon = ((iconSeparation/2)-(imageSize/2))     ## The number of pixels to center each small icon in it's "cell"
borderLeft = iconSeparation                              ## How much "white" space on the left
sizeDiff = imageSizeBig - imageSize                      ## How many pixels are there from the small icon to the big icon

## Calculate widget size
widgetWidth = (borderLeft + (iconSeparation * (numIcons+1)))
widgetHeight = imageSizeBig * 1.2

## Figure out where the bottom of the icons should be (the iconFloor)
## In this case I'm just putting them just above the bottom
iconFloor = ((widgetHeight - imageSize) / 1.2)




## Initialize a few more variables
init = 0
reset = 0

## barimageFile = "pics/osxbarback.png"
## barimage = 0
## barimageBigFile = "pics/osxbarbackBig.png"
## barimageBig = 0


def initWidget(widget):
    ## buggy in current version
    pass

def widgetUpdated(widget):
    global images
    global imagesBig
    global init
    if init == 0:
        
        ## Set the size of our widget since we're just starting
        karamba.resizeWidget(widget, widgetWidth, widgetHeight)
        
        ## Setup a background image
        ## barimage = karamba.createImage(widget, 60, 62, barimageFile)
        ## barimageBig = karamba.createImage(widget, 6, widgetHeight, barimageBigFile)
        for i in range (0,numIcons):
            images.append(karamba.createImage(widget, borderLeft + centerSmallIcon + (i*iconSeparation), iconFloor, imageFiles[i]))
            imagesBig.append(karamba.createImage(widget, borderLeft + (i*iconSeparation), widgetHeight, imageFilesBig[i]))
            karamba.createClickArea(widget, borderLeft + (i*iconSeparation), iconFloor + imageSize - imageSizeBig, iconSeparation, imageSizeBig, commands[i])
        karamba.redrawWidget(widget)
        reset = 1
        init = 1
        




#  This gets called everytime our widget is clicked.
#  Notes:
#    widget = reference to our widget
#    x = x position (relative to our widget)
#    y = y position (relative to our widget)
#    botton = button clicked:
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetClicked(widget, x, y, button):
    pass







#  This gets called everytime the mouse is moved over our widget
#  Notes
#    widget = reference to our widget
#    x = x position (relative to our widget)
#    y = y position (relative to our widget)
#    botton = button being held:
#                    0 = No Mouse Button
#                    1 = Left Mouse Button
#                    2 = Middle Mouse Button
#                    3 = Right Mouse Button, but this will never happen
#                        because the right mouse button brings up the
#                        Karamba menu.
def widgetMouseMoved(widget, x, y, button):
    global init
    global reset
    
    if init == 1:
        iconNum = -2
        
        ## If mouse is to far right or left reset everything to normal
        if ((x < (borderLeft-iconSeparation)) or (x >= (borderLeft + ((numIcons+1) * iconSeparation) ))) and reset == 0:
            if (debug):
                print "x is out of bounds.."
            resetWidget(widget)
        
        ## Or if the mouse is too low or too high reset everything to normal
        elif ((y < (iconFloor + imageSize - imageSizeBig)) or (y > (iconFloor + imageSize))) and reset == 0:
            if (debug):
                print "y is out of bounds.."
            resetWidget(widget)
        
        ## If the mouse is in the target area
        elif (x >= (borderLeft-iconSeparation)) and ((y >= (iconFloor + imageSize - imageSizeBig)) and (y <= (iconFloor + imageSize))):
            
            if (debug):
                print "processing.."
            
            ## Which icon is the mouse over?
            iconNum = (x - borderLeft) / iconSeparation   ## borderLeft is initial white-space, and iconSeparation pixels between each icon
            
            ## This returns a number between 0 and iconSeparation depending on how far through the image you are
            offset     = (x - borderLeft) % iconSeparation                  ## 0 - iconSeparation (imageSizeBig/2)
            sizeRatio = ((sizeDiff + 0.0) / imageSizeBig)                   ## Ratio of small icon to big icon
            ## This is the size of the currentIcon, and changes depending on how far through the image you are
            offsetSize = ((offset*2) * sizeRatio)                           ## 0 - sizeDiff
            
            reset = 1
            
            ## Unless the current icon comes up as a non-valid number
            if (-2 < iconNum < numIcons+1):
                
                reset = 0
                
                ## karamba.moveImage(widget, barimage, 60, 160)                
                ## karamba.moveImage(widget, barimageBig, 6, 62)                
                
                
                ################################
                ## Size and position iconNum-1
                ################################
                ## This icon:  x will increase and y will decrease with the increasing offset
                
                ## Unless we're on the first image, resize the icon to our left
                if iconNum > 0:
                    ## Calculate stuff
                    size = imageSize + (sizeDiff/2) - (offsetSize/2)
                    xpos = ( ((iconNum-2) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + ((size-imageSize)/(2*sizeRatio)) )   ## (iconSeparation/2)-(size/2) centers the image, then plus ((size-imageSize)/2), half the size we've grown
                    ypos = iconFloor - iconSeparation + offset   ## The iconSeparation variable is here because iconSeparation - offset = 0 at right border
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum-1], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum-1], xpos, ypos)
                    
                    ## Make sure the current small icon is off the screen
                    karamba.moveImage(widget, images[iconNum-1], ((iconNum-1)*iconSeparation)+imageSizeBig, widgetHeight)
                    
                    ## Make sure all the previous icons are setup normally
                    for i in range (0, iconNum - 1):
                        karamba.moveImage(widget, images[i], ((i-1)*(iconSeparation)) + borderLeft + centerSmallIcon, iconFloor)
                        karamba.moveImage(widget, imagesBig[i], ((i-1)*(iconSeparation)) + borderLeft, widgetHeight)
                
                    if (debug):
                        print "iconNum: ", iconNum-1, "\toffset: ", offset, "\toffsetSize: ", offsetSize, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                
                ################################
                ## Size and position iconNum
                ################################
                ## This icon:  x will increase and y will decrease with the increasing offset
                ##             until offset > (half of the offset), then
                ##             x will decrease and y will increase with the increasing offset
                
                if ( (iconNum > -1) and (iconNum < numIcons) ):
                    ## If we're on the left side of the icon
                    if offset < (iconSeparation/2):  ## If image is less than half of the offset
                        ## Calculate stuff
                        size = imageSize + (sizeDiff/2) + offsetSize
                        xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (iconSeparation/2) - offset )      ## Moves to the right as offset increases
                        ypos = iconFloor - iconSeparation - (offset)                                      
                        
                        ## Place stuff
                        karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                        karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                    
                    
                    ## If we're on the right side of the icon
                    else:  ## If image is greater than half of the offset
                        ## Calculate stuff
                        size = imageSize + sizeDiff + (sizeDiff/2) - offsetSize          ## the extra sizeDiff/2 is to make things look right because of the changing size
                        ## First set initial placement                  plus   center it               plus  add half of an iconSegment  and subtract the offset (0-iconSeparation pixels)
                        xpos = ( (iconNum * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) + (iconSeparation/2) - offset )
                        ## 
                        ypos = iconFloor - iconSeparation - iconSeparation + offset
                        
                        ## Place stuff
                        karamba.resizeImage(widget, imagesBig[iconNum], size, size)
                        karamba.moveImage(widget, imagesBig[iconNum], xpos, ypos)
                    
                    ## Move the current small image off the widget
                    karamba.moveImage(widget, images[iconNum], (iconNum*iconSeparation) + borderLeft + centerSmallIcon, widgetHeight)
                    
                    if (debug):
                        print "iconNum: ", iconNum, "\toffset: ", offset, "\toffsetSize: ", offsetSize, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                    
                
                
                
                
                ################################
                ## Size and position iconNum+1
                ################################
                ## This icon:  x will decrease and y will increase with the increasing offset
                
                ## Unless we're on the last image, resize the icon to our right
                if (iconNum < numIcons-1):
                    ## Calculate stuff
                    size = imageSize + (offsetSize/2)
                    xpos = ( ((iconNum+2) * iconSeparation) + borderLeft + ((iconSeparation/2)-(size/2)) - ((size-imageSize)/(2*sizeRatio)) )   ## Moves to the left as offset increases
                    ypos = iconFloor - offset
                    
                    ## Place stuff
                    karamba.resizeImage(widget, imagesBig[iconNum+1], size, size)
                    karamba.moveImage(widget, imagesBig[iconNum+1], xpos, ypos)
                    
                    ## Make sure the current small icon is off the screen
                    karamba.moveImage(widget, images[iconNum+1], ((iconNum+1)*iconSeparation)+imageSizeBig, widgetHeight)
                    
                    ## Make sure all the following icons are setup normally
                    for i in range (iconNum+2, numIcons):
                        karamba.moveImage(widget, images[i], ((i)*iconSeparation) + iconSeparation + borderLeft + centerSmallIcon, iconFloor)
                        karamba.moveImage(widget, imagesBig[i], ((i)*iconSeparation) + iconSeparation + borderLeft, widgetHeight)
                        
                    if (debug):
                        print "iconNum: ", iconNum+1, "\toffset: ", offset, "\toffsetSize: ", offsetSize, "\tsize: ", size, "\txpos: ", xpos, "\typos: ", ypos
                if (debug):
                    print
                
                
                ## Update the screen with the changes we've just made
                karamba.redrawWidget(widget)
                


## Small Function to reset the widget to it's normal state, i.e. no icons zoomed
def resetWidget(widget):
    global reset
    
    for i in range (0,numIcons):
        karamba.moveImage(widget, images[i], (i*iconSeparation) + borderLeft + centerSmallIcon, iconFloor)
        karamba.moveImage(widget, imagesBig[i], (i*iconSeparation) + borderLeft, widgetHeight)
        ## karamba.moveImage(widget, barimage, 60, 62)                
        ## karamba.moveImage(widget, barimageBig, 6, widgetHeight)                
    karamba.redrawWidget(widget)
    reset = 1



# This will be printed when the widget loads.
print "Loaded my python extension!"

